unit k2OooBExport;

interface

uses
  Classes,
  ComObj,
  Controls,
  Dialogs,
  Forms,
  JclShell,
  JclRegistry,
  kTExportApi,
  frmOpenExport,
  Registry,
  SHFolder,
  ShLwApi,
  StrUtils,
  SysUtils,
  Variants,
  VarUtils,
  Windows;

type
  EOooBase = class(Exception)
  end;

function AddDataRecord(aData: PkxpExportDataRec): LongInt; stdcall;

procedure ClearBoundDatabaseFileName; stdcall;

function CloseExport: LongInt; stdcall;

function GetDatabaseFileName: PChar; stdcall;

function GetExportInfo(var aInfo: PkxpExportInfoRec): Boolean; stdcall;

function OpenExport(aClearData: Boolean): LongInt; stdcall;

implementation

const
  cUriDatabaseContext     = 'com.sun.star.sdb.DatabaseContext';
  cUriDesktop             = 'com.sun.star.frame.Desktop';
  cUriFileContentProvider = 'com.sun.star.ucb.FileContentProvider';
  cUriInteractionHandler  = 'com.sun.star.sdb.InteractionHandler';
  cUriServiceManager      = 'com.sun.star.ServiceManager';

  mcAppName     = 'OpenOffice.org Base';
  mcAppCompany  = 'Sun Microsystems Inc.';
  mcAppVersion  = '2.4.x/3.x';
  mcDllCompany  = 'Thomas Rubein';
  mcDllAuthor   = 'Thomas Rubein';
  mcDllVersion  = '1.0';
  mcDllFileName = 'K2OOOB32.DLL';

  mcRegKeyOooBase                 = 'Software\Thomas Rubein\OpenOffice.org\Base';
  mcRegValBoundDatabaseFileName   = 'BoundDatabaseFileName';
  mcRegValUnboundDatabaseFileName = 'UnboundDatabaseFileName';

  mcDatabaseTableName         = 'ADDRESS_EXPORT';
  mcDatabaseFileNameExt       = 'odb';
  mcSuggestedDatabaseFileName = 'Adressexport.odb';

resourcestring
  msDialogCaption   = 'Datenbergabe an ' + mcAppName;
  msHeaderTextError = 'Die Datenbergabe an ' + mcAppName + ' ist fehlgeschlagen:'#13#10;
  msObjectErrorFmt  = 'Das Objekt %s kann nicht erstellt werden';

  msInvalidDatabaseConnectionError = 'Die Datenbankverbindung kann nicht erstellt werden.';
  msInvalidDatabaseContextError    = 'Der Datenbankkontext kann nicht erstellt werden.';
  msInvalidDatabaseSourceError     = 'Die Datenbankquelle kann nicht erstellt werden.';
  msInvalidServiceManagerError     = 'Der Service-Manager kann nicht geffnet werden.';
  msInvalidStarDesktop             = 'Der Desktop ist ungltig.';
  msInvalidDatabaseStatementError  = 'Die SQL-Anweisung kann nicht erstellt werden.';

var
  mOooDatabaseConnection: Variant;
  mOooDatabaseContext   : Variant; 
  mOooDatabaseSource    : Variant; 
  mOooInteractionHandler: Variant; 
  mOooServiceManager    : Variant; 
  mOooStarDesktop       : Variant; 
  
  mBoundDatabaseFileName: string;
  
  mUnboundDatabaseFileName: string;
  
  mDatabaseFileNamePtr: PString;

function InquireForDatabaseFileName: Boolean; forward;

function GetFileUrl(const aPath: string): string; forward;

procedure CheckValidPointer(const aPointer: Pointer; const aErrorText: string); forward;

procedure CheckValidVariant(const aVariant: Variant; const aErrorText: string); forward;

procedure DisposeAndClearAllVariants; forward;

procedure CreateDatabaseTable; forward;

function CheckedCreateUnoService(const aUri: String): Variant; forward;

function GetEmptyVarArrayParam: Variant; forward;

function GetDatabaseSourceAndConnection(aClearData: Boolean): Boolean; forward;

function IsOpenOfficeOrgInstalled: Boolean; forward;

function IsValidVariant(const aValue: Variant): Boolean; forward;

procedure LoadDatabaseFileNames; forward;

function NormalizeDatabaseFileName(const aFileName: string): string; forward;

procedure SaveDatabaseFileNames; forward;

function AddDataRecord(aData: PkxpExportDataRec): LongInt;
  
  function GetSqlString(const aValue: PAnsiChar): string;
  begin
    if Assigned(aValue) and (StrLen(aValue) > 0) then
      Result := AnsiQuotedStr(aValue, #39)
    else
      Result := 'NULL';
  end;

resourcestring
  sText01 = 'Der Datenverweis ist ungltig (Nullzeiger).';
const
  cSqlFmt = 'INSERT INTO "' + mcDatabaseTableName + '" ('+
            '"Anrede", "Titel", "Vorname", "Name", "Strasse", "Zusatz", ' +
            '"PLZ", "Ort", "Art", "Vorwahl", "Telefon") ' +
            'VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)';
var
  lOooQuery: Variant;
  lSql     : string;
begin
  try
    CheckValidPointer(aData, sText01);
    CheckValidVariant(mOooDatabaseConnection, msInvalidDatabaseConnectionError);

    lOooQuery := mOooDatabaseConnection.CreateStatement;
    CheckValidVariant(lOooQuery, msInvalidDatabaseStatementError);

    lSql := Format(cSqlFmt, [GetSqlString(aData^.Anrede),
                             GetSqlString(aData^.Titel),
                             GetSqlString(aData^.Vorname),
                             GetSqlString(aData^.Name),
                             GetSqlString(aData^.Strasse),
                             GetSqlString(aData^.Zusatz),
                             GetSqlString(aData^.PLZ),
                             GetSqlString(aData^.Ort),
                             GetSqlString(aData^.TelZusatz),
                             GetSqlString(aData^.Vorwahl),
                             GetSqlString(aData^.Telefon)]);

    lOooQuery.ExecuteUpdate(lSql);
    Result    := kxpErrorSuccess;
    lOooQuery := Unassigned;
  except
    on E: Exception do
    begin
      Result := kxpErrorAppendData;
      Application.MessageBox(PChar(msHeaderTextError + E.Message), PChar(msDialogCaption), MB_OK + MB_ICONSTOP);
    end;
  end;
end;

function InquireForDatabaseFileName: Boolean;
var
  lForm: TOpenExportForm;
begin
  mDatabaseFileNamePtr := nil;
  lForm := TOpenExportForm.Create(nil);
  try
    lForm.DatabaseFileName := mUnboundDatabaseFileName;
    if IsPositiveResult(lForm.ShowModal) and
      (Length(ExtractFileName(lForm.DatabaseFileName)) > 0) then
    begin
      mUnboundDatabaseFileName := NormalizeDatabaseFileName(lForm.DatabaseFileName);
      mDatabaseFileNamePtr     := Addr(mUnboundDatabaseFileName);
      mBoundDatabaseFileName   := IfThen(lForm.IsBoundDatabaseFileName, mDatabaseFileNamePtr^, '');
    end;
  finally
    lForm.Release;
  end;
  Result := Assigned(mDatabaseFileNamePtr);
end;

function GetFileUrl(const aPath: string): string;
var
  lProvider: Variant;
begin
  lProvider := CheckedCreateUnoService(cUriFileContentProvider);
  Result    := lProvider.GetFileUrlFromSystemPath('', aPath);
  lProvider := Unassigned;
end;

function CheckedCreateUnoService(const aUri: String): Variant;
begin
  CheckValidVariant(mOooServiceManager, msInvalidServiceManagerError);
  Result:= mOooServiceManager.createInstance(aUri);
  CheckValidVariant(Result, Format(msObjectErrorFmt, [aUri]));
end;

procedure CheckValidPointer(const aPointer: Pointer; const aErrorText: string);
begin
  if not Assigned(aPointer) then
    raise EOooBase.Create(aErrorText);
end;

procedure CheckValidVariant(const aVariant: Variant; const aErrorText: string);
begin
  if not IsValidVariant(aVariant) then
    raise EOooBase.Create(aErrorText);
end;

procedure DisposeAndClearAllVariants;
begin
  if not VarIsEmpty(mOooDatabaseConnection) then
  begin
    mOooDatabaseConnection.Flush;
    mOooDatabaseConnection.Close;
    mOooDatabaseConnection.Dispose;
    mOooDatabaseConnection := Unassigned;
  end;

  if not VarIsEmpty(mOooDatabaseSource) then
  begin
    mOooDatabaseSource.Dispose;
    mOooDatabaseSource := Unassigned;
  end;

  if not VarIsEmpty(mOooDatabaseContext) then
  begin
    mOooDatabaseContext := Unassigned;
  end;

  if not VarIsEmpty(mOooStarDesktop) then
  begin
    mOooStarDesktop := Unassigned;
  end;

  if not VarIsEmpty(mOooServiceManager) then
  begin
    mOooServiceManager := Unassigned;
  end;
end;

procedure ClearBoundDatabaseFileName;
begin
  mBoundDatabaseFileName := '';
  SaveDatabaseFileNames;
end;

function CloseExport: LongInt;
begin
  DisposeAndClearAllVariants;
  SaveDatabaseFileNames;
  Result := kxpErrorSuccess;
end;

procedure CreateDatabaseTable;
const
  cSql = 'CREATE TABLE "' + mcDatabaseTableName + '" (' +
         '"ID" INTEGER GENERATED BY DEFAULT AS IDENTITY (START WITH 1) NOT NULL PRIMARY KEY, ' +
         '"Anrede"  VARCHAR(255) NULL,' +
         '"Titel"   VARCHAR(255) NULL,' +
         '"Vorname" VARCHAR(255) NULL,' +
         '"Name"    VARCHAR(255) NULL,' +
         '"Strasse" VARCHAR(255) NULL,' +
         '"Zusatz"  VARCHAR(255) NULL,' +
         '"PLZ"     VARCHAR(255) NULL,' +
         '"Ort"     VARCHAR(255) NULL,' +
         '"Art"     VARCHAR(255) NULL,' +
         '"Vorwahl" VARCHAR(255) NULL,' +
         '"Telefon" VARCHAR(255) NULL)';
var
  lOooQuery: Variant;
begin
  CheckValidVariant(mOooDatabaseConnection, msInvalidDatabaseConnectionError);
  lOooQuery := mOooDatabaseConnection.CreateStatement;
  CheckValidVariant(lOooQuery, msInvalidDatabaseStatementError);

  lOooQuery.ExecuteUpdate(cSql);
end;

function GetDatabaseFileName: PChar;
begin
  if Assigned(mDatabaseFileNamePtr) and (Length(mDatabaseFileNamePtr^) > 0) then
    Result := PChar(mDatabaseFileNamePtr^)
  else
    Result := nil;
end;

function GetDatabaseSourceAndConnection(aClearData: Boolean): Boolean;
var
  lOooQuery : Variant;
  lOooResult: Variant;
begin
  CheckValidVariant(mOooDatabaseContext, msInvalidDatabaseContextError);
  Result := False;

  if Length(mBoundDatabaseFileName) > 0 then
    mDatabaseFileNamePtr := Addr(mBoundDatabaseFileName)
  else
    mDatabaseFileNamePtr := nil;
  
  if Assigned(mDatabaseFileNamePtr) and FileExists(mDatabaseFileNamePtr^) then
    
    mOooDatabaseSource := mOooDatabaseContext.GetByName(
                          GetFileUrl(mDatabaseFileNamePtr^))
  else if Assigned(mDatabaseFileNamePtr) or InquireForDatabaseFileName then
  begin
    
    Assert(Assigned(mDatabaseFileNamePtr));
    if FileExists(mDatabaseFileNamePtr^) then
      mOooDatabaseSource := mOooDatabaseContext.GetByName(
                            GetFileUrl(mDatabaseFileNamePtr^))
    else begin
      
      mOooDatabaseSource     := mOooDatabaseContext.CreateInstance;
      mOooDatabaseSource.Url := 'sdbc:embedded:hsqldb';
      mOooDatabaseSource.DatabaseDocument.StoreAsUrl(
        GetFileUrl(mDatabaseFileNamePtr^), GetEmptyVarArrayParam);
    end;
  end else
    
    Exit;

  CheckValidVariant(mOooDatabaseSource, msInvalidDatabaseSourceError);
  
  if not IsValidVariant(mOooDatabaseConnection) then
  begin
    if mOooDatabaseSource.IsPasswordRequired then
    begin
      mOooInteractionHandler := CheckedCreateUnoService(cUriInteractionHandler);
      mOooDatabaseConnection := mOooDatabaseSource.ConnectWithCompletion(mOooInteractionHandler);
    end else
      mOooDatabaseConnection := mOooDatabaseSource.GetConnection('', '');
  end;
  CheckValidVariant(mOooDatabaseConnection, msInvalidDatabaseConnectionError);

  lOooQuery := mOooDatabaseConnection.CreateStatement;
  CheckValidVariant(lOooQuery, msInvalidDatabaseStatementError);
  
  if aClearData then
    try
      
      lOooQuery.ExecuteQuery('DROP TABLE ' + mcDatabaseTableName);
    except;
    end;
  try
    
    lOooResult := lOooQuery.ExecuteQuery('SELECT COUNT (*) FROM ' + mcDatabaseTableName);
  except
    
    CreateDatabaseTable;
  end;
  Result := True;
end;

function GetEmptyVarArrayParam: Variant;
begin
  Result:= VarArrayCreate([0, -1], varVariant);
end;

function GetExportInfo(var aInfo: PkxpExportInfoRec): Boolean;
begin
  Result := False;
  if Assigned(aInfo) then
  begin
    LoadDatabaseFileNames;

    ZeroMemory(aInfo, SizeOf(aInfo));
    aInfo^.InterfaceVerMajor  := 1;
    aInfo^.InterfaceVerMinor  := 0;
    aInfo^.AppName            := mcAppName;
    aInfo^.AppCompany         := mcAppCompany;
    aInfo^.AppVersion         := mcAppVersion;
    aInfo^.AppExportName      := mcAppName;
    aInfo^.IsFileNameRequired := True;
    aInfo^.DefaultFileExt     := mcDatabaseFileNameExt;
    aInfo^.DllName            := mcDllFileName;
    aInfo^.DllCompany         := mcDllCompany;
    aInfo^.DllAuthor          := mcDllAuthor;
    aInfo^.DllVersion         := mcDllVersion;
    aInfo^.IsClearSupported   := True;
    aInfo^.IsAppInstalled     := IsOpenOfficeOrgInstalled;
    aInfo^.LastFileName       := GetDatabaseFileName;
    Result := True;
  end;
end;

function IsOpenOfficeOrgInstalled: Boolean;
const
  cRegKey = 'SOFTWARE\Microsoft\Windows\CurrentVersion\App Paths\soffice.exe';
begin
  Result := FileExists(RegReadStringDef(HKLM, cRegKey, '', ''));
end;

function IsValidVariant(const aValue: Variant): Boolean;
begin
  Result:= not (VarIsEmpty(aValue) or VarIsNull(aValue) or VarIsClear(aValue));
end;

procedure LoadDatabaseFileNames;
begin
  mBoundDatabaseFileName := NormalizeDatabaseFileName(
    RegReadStringDef(HKCU, mcRegKeyOooBase, mcRegValBoundDatabaseFileName, ''));

  mUnboundDatabaseFileName := NormalizeDatabaseFileName(
    RegReadStringDef(HKCU, mcRegKeyOooBase, mcRegValUnboundDatabaseFileName, ''));
  if Length(mUnboundDatabaseFileName) = 0 then
    mUnboundDatabaseFileName := NormalizeDatabaseFileName(mcSuggestedDatabaseFileName);

  mDatabaseFileNamePtr := Addr(mBoundDatabaseFileName);
end;

function NormalizeDatabaseFileName(const aFileName: string): string;
var
  lFileName: string;
  lFileDir: string;
begin
  Result := aFileName;
  if Length(aFileName) > 0 then
  begin
    lFileName := ExtractFileName(aFileName);
    lFileDir  := ExtractFileDir(aFileName);
    if (Length(lFileDir) = 0) and (Length(lFileName) > 0) then
      Result := ChangeFilePath(lFileName, GetSpecialFolderLocation(CSIDL_PERSONAL));
    if Length(ExtractFileExt(Result)) = 0 then
      Result := ChangeFileExt(Result, '.' + mcDatabaseFileNameExt);
  end;
end;

function OpenExport(aClearData: Boolean): LongInt;
begin
  LoadDatabaseFileNames;
  try
    Screen.Cursor:= crHourglass;
    try
      Application.ProcessMessages;
      
      if not IsValidVariant(mOooServiceManager) then
      begin
        mOooServiceManager := CreateOleObject(cUriServiceManager);
        CheckValidVariant(mOooServiceManager, Format(msObjectErrorFmt, [cUriServiceManager]));
      end;
      
      if not IsValidVariant(mOooStarDesktop) then
        mOooStarDesktop := CheckedCreateUnoService(cUriDesktop);
      
      if not IsValidVariant(mOooDatabaseContext) then
        mOooDatabaseContext := CheckedCreateUnoService(cUriDatabaseContext);
      
      if GetDatabaseSourceAndConnection(aClearData) then
      begin
        SaveDatabaseFileNames;
        Result := kxpErrorSuccess;
      end else
        Result := kxpErrorInitialization;
    finally
      Screen.Cursor:= crDefault;
    end;
  except
    on E: Exception do
    begin
      DisposeAndClearAllVariants;
      Result := kxpErrorInitialization;
      Application.MessageBox(PChar(msHeaderTextError + E.Message), PChar(msDialogCaption), MB_OK + MB_ICONSTOP);
    end;
  end;
end;

procedure SaveDatabaseFileNames;

  procedure WriteToRegistry(const aRegValue, aValue: string);
  begin
    if Length(aValue) > 0 then
      RegWriteString(HKCU, mcRegKeyOooBase, aRegValue, aValue)
    else if RegValueExists(HKCU, mcRegKeyOooBase, aRegValue) then
      RegDeleteEntry(HKCU, mcRegKeyOooBase, aRegValue);
  end;

begin
  WriteToRegistry(mcRegValBoundDatabaseFileName,   mBoundDatabaseFileName);
  WriteToRegistry(mcRegValUnboundDatabaseFileName, mUnboundDatabaseFileName);
end;

initialization
  LoadDatabaseFileNames;

finalization
  SaveDatabaseFileNames;

end.
