unit k2OooB32TestCases;

interface

uses
  Classes,
  Controls,
  Forms,
  k2OooB32Dll,
  kTExportApi,
  TestFramework,
  Variants,
  Windows;

type
  TTestGetExportInfo = class(TTestCase)
  published
    procedure TestGetExportInfo;
    procedure TestGetExportInfoNilPointer;
  end;

  TTestOpenCloseExport = class(TTestCase)
  published
    procedure TestClearDatabaseFileName;
    procedure TestOpenExport;
    procedure TestCloseExport;
  end;

  TTestAddDataRecords = class(TTestCase)
  protected
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestAddValidDataRecords;
    procedure TestAddDataRecordNilPointer;
  end;

  TTestClearTableAndAddDataRecords = class(TTestAddDataRecords)
  protected
    procedure SetUp; override;
  end;

implementation

const
  mcDataRecords: array[0..2] of TkxpExportDataRec = (
    (
    
    Geschlecht : '';
    Anrede     : '';
    Firma      : '';
    Name       : '';
    Vorname    : '';
    Titel      : '';
    Adelstitel : '';
    NameVorsatz: '';
    NameZusatz : '';
    Zusatz     : '';
    PLZ        : '';
    Ort        : '';
    Ortsteil   : '';
    AlOrt      : 0;
    Strasse    : '';
    Hausnummer : '';
    Land       : '';
    Vorwahl    : '';
    TelZusatz  : '';
    Telefon    : '';
    WWW        : '';
    Email      : ''),
    (
    
    Geschlecht : '???????';
    Anrede     : '?????????';
    Firma      : '???????????????????????????????????????????????????????????????????????????????????????????????????????????????????????';
    Name       : '???????????????????????????????????????????????????????????????????????????????????????????????????????????????????????';
    Vorname    : '?????????????????????????????';
    Titel      : '?????????????????????????????';
    Adelstitel : '?????????????????????????????';
    NameVorsatz: '?????????????????????????????';
    NameZusatz : '?????????????????????????????';
    Zusatz     : '???????????????????????????????????????????????????????????????????????????????????????????????????????????????????????';
    PLZ        : '?????????';
    Ort        : '?????????????????????????????????????????????????????????????????????';
    Ortsteil   : '???????????????????????????????????????';
    AlOrt      : 0;
    Strasse    : '?????????????????????????????????????????????';
    Hausnummer : '?????????';
    Land       : '??';
    Vorwahl    : '?????????';
    TelZusatz  : '??????????????';
    Telefon    : '???????????????????';
    WWW        : '???????????????????????????????????????????????????????????';
    Email      : '???????????????????????????????????????????????????????????'),
    (

    Geschlecht : 'Geschl.';
    Anrede     : 'Anrede';
    Firma      : 'Firma';
    Name       : 'Name';
    Vorname    : 'Vorname';
    Titel      : 'Titel';
    Adelstitel : 'Adelstitel';
    NameVorsatz: 'NameVorsatz';
    NameZusatz : 'NameZusatz';
    Zusatz     : 'Zusatz';
    PLZ        : 'PLZ';
    Ort        : 'Ort';
    Ortsteil   : 'Ortsteil';
    AlOrt      : 0;
    Strasse    : 'Strasse';
    Hausnummer : 'Hausnummer';
    Land       : 'DE';
    Vorwahl    : 'Vorwahl';
    TelZusatz  : 'TelZusatz';
    Telefon    : 'Telefon';
    WWW        : 'WWW';
    Email      : 'Email'));

procedure TTestGetExportInfo.TestGetExportInfo;
var
  lInfo: TkxpExportInfoRec;
  P: PkxpExportInfoRec;
begin
  ZeroMemory(@lInfo, SizeOf(lInfo));
  P := @lInfo;
  CheckTrue(GetExportInfo(P));
  CheckEqualsString('OpenOffice.org Base', P^.AppName);
end;

procedure TTestGetExportInfo.TestGetExportInfoNilPointer;
var
  P: PkxpExportInfoRec;
begin
  P := nil;
  CheckFalse(GetExportInfo(P));
end;

procedure TTestOpenCloseExport.TestClearDatabaseFileName;
begin
  ClearDatabaseFileName;
  Check(GetDatabaseFileName = nil);
end;

procedure TTestOpenCloseExport.TestOpenExport;
begin
  Check(OpenExport(True) = kxpErrorSuccess);
end;

procedure TTestOpenCloseExport.TestCloseExport;
begin
  Check(CloseExport = kxpErrorSuccess);
end;

procedure TTestAddDataRecords.SetUp;
begin
  OpenExport(True);
end;

procedure TTestAddDataRecords.TearDown;
begin
  CloseExport;
end;

procedure TTestAddDataRecords.TestAddValidDataRecords;
var
  I: Integer;
begin
  for I := 0 to High(mcDataRecords) do
    Check(AddDataRecord(@mcDataRecords[I]) = kxpErrorSuccess);
end;

procedure TTestAddDataRecords.TestAddDataRecordNilPointer;
begin
  Check(AddDataRecord(nil) = kxpErrorAppendData);
end;

procedure TTestClearTableAndAddDataRecords.SetUp;
begin
  OpenExport(False);
end;

initialization
  RegisterTest(TTestGetExportInfo.Suite);
  RegisterTest(TTestOpenCloseExport.Suite);
  RegisterTest(TTestAddDataRecords.Suite);
  RegisterTest(TTestClearTableAndAddDataRecords.Suite);
end.


